#include "loadModule.h"

#include "libxml/parser.h"
#include "type.h"

#include "assert.h"

loadModule* loadModule::m_loadMobule = NULL;


int loadModule::handlerItemAttribute(xmlNode* xml_item, Item* item){
	char* str_name = (char *)xmlGetProp(xml_item, (xmlChar*)STRINGRC_ATTRIBUTE_NAME);
	char* str_content = (char *)xmlNodeGetContent(xml_item);

	if( (str_name != NULL) && (str_content != NULL) ){
		item->addAttribute(str_name, str_content);
		MessageBoxA(NULL,str_name, str_content,MB_OK);
		return OK;
	}

	return FAIL;
}


int loadModule::handlerItemEvent(xmlNode* xml_item, Item* item){
	char* str_method = (char *)xmlGetProp(xml_item, (xmlChar*)STRINGRC_ATTRIBUTE_METHOD);
	char* str_code = (char *)xmlNodeGetContent(xml_item);

	if( (str_method != NULL) && (str_code != NULL) ){
		DWORD nMethod = atoi(str_method);
		item->addevent(nMethod, str_code);
		MessageBoxA(NULL,str_method, str_code,MB_OK);
		return OK;
	}

	return FAIL;
}


int loadModule::handlerWindow(xmlNode* xml_window){

	char* str_class = (char *)xmlGetProp(xml_window, (xmlChar*)STRINGRC_ATTRIBUTE_CLASS);
	if(str_class == NULL){
		return FAIL;
	}
	
	loadModule *module = loadModule::getLoadModule();
	Window* win = new Window();

	module->m_windows[str_class] = win;

	for(xmlNode* node = xml_window->children; node != NULL; node = node->next){

		if(node->type != XML_ELEMENT_NODE){
			continue;
		}
		
		if((node->name != NULL) && (strcmp((char*)node->name, STRINGRC_NODE_ITEM) == 0)){
			handlerItem(node, win);
		}
	}

	return OK;
}

int loadModule::handlerItem(xmlNode* xml_item, Window *window){
	
	char* str_id = (char *)xmlGetProp(xml_item, (xmlChar*)STRINGRC_ATTRIBUTE_ID);
	char* str_type = (char *)xmlGetProp(xml_item, (xmlChar*)STRINGRC_ATTRIBUTE_TYPE);

	if((str_id == NULL) || (str_type == NULL)){
		assert(false);//Item忡 ID TYPE  ʽϴ.
		return FAIL;
	}
	
	DWORD nID = atoi(str_id);
	
	Item* item = new Item();
	window->addItem(nID, item);
	
	if( ((string)str_type) == "base" ){
		item->setType(ITEM_TYPE_BASE);
	}else if( ((string)str_type) == "create" ){
		item->setType(ITEM_TYPE_CREATE);
	}else{
		assert(false);//Item忡 TYPE Դϴ.
		item->setType(ITEM_TYPE_BASE);
	}

	for(xmlNode* node = xml_item->children; node != NULL; node = node->next){

		if((node->type != XML_ELEMENT_NODE) || (node->name == NULL)){
			continue;
		}
		
		//MessageBox(NULL, (char*)node->name, "", MB_OK);
		if((strcmp((char*)node->name, STRINGRC_NODE_ATTRIBUTE) == 0)){
			handlerItemAttribute(node, item);

		}else if((strcmp((char*)node->name, STRINGRC_NODE_EVENT) == 0)){
			handlerItemEvent(node, item);
		}
	}
	

	return OK;
}


int loadModule::handlerApplication(xmlNode* xml_app){
	for(xmlNode* node = xml_app->children; node != NULL; node = node->next){

		if(node->type != XML_ELEMENT_NODE){
			continue;
		}
		
		if((node->name != NULL) && (strcmp((char*)node->name, STRINGRC_NODE_WINDOW) == 0)){
			handlerWindow(node);
		}
		
	}
	return OK;
}

int loadModule::parsing(string str_xmlpath){
	
	xmlDoc* doc = xmlReadFile(str_xmlpath.c_str(), NULL, 0);
	
	DWORD ret = FAIL;
	
	if(doc == NULL){
		//ASSERT(false);//Ľ  XML ջǾ - ڵ  Ȯ
		MessageBox(NULL, MESSAGE_FAIL_LOAD, CAPTION_ERROR_EXIT, MB_OK);
		return DEF_ERROR_XML_LOAD_FAIL;
	}
	
	xmlNode* root_element = xmlDocGetRootElement(doc);
	if(root_element == NULL){
		//ASSERT(false);//XML 
		MessageBox(NULL, MESSAGE_FAIL_LOAD, CAPTION_ERROR_EXIT, MB_OK);
		return DEF_ERROR_XML_EMPTY;
	}

	for(xmlNode* node = root_element; node != NULL; node = node->next){
		
		if(node->type != XML_ELEMENT_NODE){
			continue;
		}

		if((node->name != NULL) && (strcmp((char*)node->name, STRINGRC_NODE_ROOT) == 0)){
			handlerApplication(node);
		}
		
	}

	return ret;
}

loadModule* loadModule::getLoadModule(){
	if(m_loadMobule != NULL){
		return m_loadMobule;
	}

	m_loadMobule = new loadModule();

	return m_loadMobule;
}